"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var fs_extra_1 = require("fs-extra");
var path_1 = require("path");
var DEFAULT_FILE_NAME = "data.json";
var FileStorage = (function () {
    function FileStorage(dataPath) {
        this.dataPath = dataPath;
        this.cachedStorage = {};
        this.filePath = path_1.join(dataPath, DEFAULT_FILE_NAME);
        try {
            this.cachedStorage = fs_extra_1.readJsonSync(this.filePath);
        }
        catch (err) {
            if (err.code !== 'ENOENT') {
                throw err;
            }
        }
    }
    FileStorage.prototype.get = function (key) {
        return this.cachedStorage[key];
    };
    FileStorage.prototype.set = function (key, value) {
        this.cachedStorage[key] = value;
        fs_extra_1.writeJSONSync(this.filePath, this.cachedStorage);
    };
    FileStorage.prototype.remove = function (key) {
        delete this.cachedStorage[key];
        fs_extra_1.writeJSONSync(this.filePath, this.cachedStorage);
    };
    return FileStorage;
}());
exports.default = FileStorage;
//# sourceMappingURL=FileStorage.js.map